/* Copyright (c) 2008 Telefónica Investigación y Desarrollo SAU.

This file is part of LinuxMD-XOS. LinuxMD-XOS is part of XtreemOS, a Linux-based Grid Operating System, see <http://www.xtreemos.eu> for more details. The XtreemOS project has been developed with the financial support of the European Commission's IST program under contract #FP6-033576.

LinuxMD-XOS is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 2 of the License, or (at your option) any later version.

LinuxMD-XOS is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with LinuxMD-XOS. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _INCLUDE_XOS_CREDAGENT_H
#define _INCLUDE_XOS_CREDAGENT_H

/**
 ** @file  xos_credagent.h
 ** @brief libxoscredagent interface
 **
 ** This file should be included by any application that uses libxos-credagent
 ** and by credagent/creduiagent modules. 
 **/

// API for applications
//
/** @brief get credential invoking the credagent module associated to 
 ** configuration_name to get it.
 ** @param[in] configuration_name the configuration name used to obtain the
 **            credential. Library searches /etc/xos/creds/<configuration_name>.conf
 **            to known the credagent module to use.
 ** @retval string with the credential. This string must be freed by caller.
 ** @retval NULL if failure
 **
 ** This function is used to provide a plug-able, modular system to get credentials. 
 ** Library objective is to allow administrators to change the method used by
 ** applications to get the credential to authenticate in a Single Sign-On (SSO) 
 ** infrastructure, without source code modification.
 **
 ** When a application invokes this function, library searches file 
 ** /etc/xos/creds/<configuration_name>.conf to know the credagent module needed
 ** to get the credential. Module is load with dlopen and module function
 ** credagent_getcred is called.
 **
 ** Note that /etc/xos/creds directory is protected: only root user or users
 ** member of group "credagent" are allowed to access this directory. This
 ** implies that only applications running as root or with SGID "credagent" may
 ** call this function. This is not a problem, because final user applications
 ** should not use libxos-credagent but libxos-getcred, that invokes a
 ** SGID wrapper, runcredagent, that calls this function. In XtreemOS,
 ** runcredagent is a link to startxtreemos.

 ** In XtreemOS, credential is expected to be a private key and X.509
 ** certificate in PEM format, but this function does not check it, because
 ** other applications may use the library to use other type of credentials
 ** (e.g. passphrases).
 **/
 char *xos_credagent_getcred(char *configuration);

// utilities: set environment with variables from section
/** @brief utility function to set configuration parameters from a section in 
** environment variables.
 ** @param[in] config_name the configuration name used to obtain the
 **            credential and read configuration parameters. Library parses file
 **            /etc/xos/creds/<configuration_name>.conf 
 ** @param[in] section the name of the section in configuration file to parse
 ** @retval zero on success. 
 ** @retval -1 on error.
 **
 ** This helper function allows applications to read parameters from a credential
 ** configuration file. Sections general, credagent and creduiagent are used by the
 ** library, but application may use its own sections to get configuration data.
 ** For example, startxtreemos applications use "xtreemfs" section to obtain
 ** configuration about mounting a XtreemFS volume when running.
 ** Afer this call, application may read values ussing getenv("key");
 **/
int xos_setconfigenv(char *config_name,char *section);

/** @brief utility function to get configuration sections
 ** @param[in] config_name the configuration name used to obtain the
 **            credential and read configuration parameters. Library parses file
 **            /etc/xos/creds/<configuration_name>.conf 
 ** @retval a array of char * with the section names. This data must be freed by the caller.
 ** @retval -NULL on error.
 **
 ** This helper function allows applications to get sections names from a credential
 ** configuration file. Sections general, credagent and creduiagent are used by the
 ** library, but application may use its own sections to get configuration data.
 ** For example, startxtreemos applications use "xtreemfs" section to obtain
 ** configuration about mounting a XtreemFS volume when running.
 ** Application may use xos_setconfigenv to read the parameters for a particular section.
 **/
char **xos_readconfigsections(char *config_name);

// API for credagent modules
//
typedef void * CREDAGENT_HANDLE;

/** @brief function to obtain a handle needed to use libxos-credagent API for
 **        credagent modules
 ** @param[in] config_name the configuration name specified by application when
 **            invoked xos_credagent_getcred and received by module as parameter
               in credagent_getcred function.
 ** @param[in] name is a pointer to string, that will be filled by the library
 **            with the name of the credagent module. This string must be freed
 **            by the caller when not needed anymore. This parameter may be NULL
 **            if module does not need this information.
 ** @retval handle that must be used with most libxos_credagent API calls. Handler
 **         should be freed with xos_credagentso_destroy when not needed anymore.
 **
 ** This function is needed by all credagent modules, because the obtained handler 
 ** is mandatory is most libxos-credagent calls. Handler must be freed using
 ** xos_credagentso_instance when not needed anymore.
 **
 **/
CREDAGENT_HANDLE xos_credagentso_instance(char *config_name,char **name);

/** @brief function to freed the handle obtained with xos_credagentso_instance 
 **        when not needed anymore.
 ** @param[in] handle obtained from a previous call to xos_credagentso_instance
 ** @retval this function does not return anything.
 **
 ** Any credagent module that use libxos-credagent API, must first obtain a handle
 ** with xos_credagentso_instance and use this handle with the other functions.
 ** When handler is not needed anymore, module must call this function to free
 ** resources.
 **/
void xos_credagentso_destroy(CREDAGENT_HANDLE handle);

/** @brief get a parameter value from "credagent" section in configuration file
 ** @param[in] handle of credagent module. This handle may be obtained
 ** invoking xos_credagentso_instance and must be destroyed when not needed
 ** anymore with xos_credagentso_destroy.
 ** @param[in] key the name of the parameter to retrieve from credagent section.
 ** @retval string value of the requested parameter on success. The value must be
 **         freed by caller when not needed anymore.
 ** @retval NULL on error
 **
 ** This function is used by credagent modules to obtain a credagent parameter 
 ** from credential configuration name. 
 **/
char *xos_credagentso_getparameter(CREDAGENT_HANDLE handle,char *key);


// API for credagent/creduiagent modules
//
struct creduiagent_handle;
typedef struct creduiagent_handle * CREDUIAGENT_HANDLE;
/** @brief function to obtain a handle needed to use libxos-credagent API for
 **        interaction with the creduiagent module specified in the credential
 **        configuration file.
 ** @param[in] config_name the credential configuration file used to know the
 **            creduiagent module is /etc/xos/creds/<config_name>.conf
 ** @param[in] name is a pointer to string, that will be filled by the library
 **            with the name of the creduiagent module. This string must be freed
 **            by the caller when not needed anymore. This parameter may be NULL
 **            if module does not need this information.
 ** @retval handle that must be used with most libxos_creduiagent API calls. Handler
 **         should be freed with xos_creduiagent_destroy when not needed anymore.
 **
 ** This function is needed by every credagent module that needs to interact with
 ** the creduiagent module specified in the credential configuration name, because
 ** the obtained handler is mandatory is most related calls. Handler must be freed using
 ** xos_credauigentso_instance when not needed anymore.
 **/
CREDUIAGENT_HANDLE xos_creduiagent_instance(char *config_name,char **name);

/** @brief function to freed the handle obtained with xos_creduiagentso_instance 
 **        when not needed anymore.
 ** @param[in] handle obtained from a previous call to xos_creduiagentso_instance
 ** @retval this function does not return anything.
 **
 ** Any credagent module that use libxos-credagent API to interact with the creduiagent
 ** specified in the credential configuration name, must first obtain a handle
 ** with xos_creduiagent_instance and use this handle with the other functions.
 ** When handler is not needed anymore, module must call this function to free
 ** resources.
 **
 ** Warning: creduiagent modules receives a CREDUIAGENT_HANDLE in each call, but they
 ** must not freed the handler using this function, because that handler is owned by
 ** credagent module, not by creduiagent module.
 **/
void xos_creduiagent_destroy(CREDUIAGENT_HANDLE handle); 

/** @brief Use creduiagent module to ask user a login and password or a password only.
 **
 ** @param[in] handle of creduiagent module. This handle may be obtained
 ** invoking xos_creduiagent_instance and must be destroyed when not needed
 ** anymore with xos_creduiagent_destroy. Handle may be also received as
 ** parameter in a creduiagent modulue function. In last case, don't 
 ** destroy the handle, because handle owner is the caller, not the
 ** creduiagent module.
 ** @param[in] message the informative text to display user when asking
 **      login/password. Pass NULL if you want that the creduiagent shows its
 **      default message (perhaps a message specified in the configuration file)
 ** @param[in] login pointer to a string that (if not NULL) will be filled
 **            with the username typed by the user. This string must be freed
               by the caller when not needed anymore. If parameter is NULL,
 **            only PIN number is requested. If (*login) is not NULL, the
 **            value is toked as the initial value of login, but user may
 **            change the value.
 ** @param[in] pin pointer to a string that will be filled with the password
 **            typed by the user. This string must be freed by the caller when
 **            no needed anymore.
 ** @param[in] length the suggested maximum length of the password. Warning:
 **            Some creduiagent modules may ignore this parameter. Consider
 **            this value as a hint, not as a validation.
 ** @param[in] retry pass 1 in this parameter if this is invocation is a
 **            retry after user provided a bad password, pass 0 otherwise.
 **            This is only a hint, some creduiagent modules may use
 **            this value, others may ignore it. 
 ** @retval 0 on success
 **         -1 on error
 **
 ** This function is used to ask a username and password (or only the password if
 ** parameter login is NULL). A creduiagent module may not interact with a user
 ** actually, and for example read this values from configuration file.
 **/
int xos_creduiagent_ask_login_password(CREDUIAGENT_HANDLE handle,char *message,
		char **login,char **password,int max_length,char retry) ;
/** @brief Use creduiagent module to ask user a PIN number and (optionally) a
 **        username.
 ** @param[in] handle of creduiagent module. This handle may be obtained
 ** invoking xos_creduiagent_instance and must be destroyed when not needed
 ** anymore with xos_creduiagent_destroy. Handle may be also received as
 ** parameter in a creduiagent modulue function. In last case, don't 
 ** destroy the handle, because handle owner is the caller, not the
 ** creduiagent module.
 ** @param[in] message the informative text to display user when asking the
 **      PIN. Pass NULL if you want that the creduiagent shows its default
 **      message (perhaps a message specified in the configuration file)
 ** @param[in] login pointer to a string that (if not NULL) will be filled
 **            with the username typed by the user. This string must be freed
               by the caller when not needed anymore. If parameter is NULL,
 **            only PIN number is requested. If (*login) is not NULL, the
 **            value is toked as the initial value of login, but user may
 **            change the value.
 ** @param[in] pin pointer to a string that will be filled with the PIN number
 **            typed by the user. This string must be freed by the caller when
 **            no needed anymore.
 ** @param[in] length the number of PIN digits. Warning:
 **            Some creduiagent modules may ignore this parameter. Consider
 **            this value as a hint, not as a validation.
 ** @param[in] retry pass 1 in this parameter if this is invocation is a
 **            retry after user provided a bad password, pass 0 otherwise.
 **            This is only a hint, some creduiagent modules may use
 **            this value, others may ignore it. Values 2 and 3 are special:
 **            value 2 is received after bad PIN if no more PIN tries are 
 **            allowed and a full password is required as PUK. Value 3 is the
 **            same than value 2, except that authentication was rejected
 **            without checking PIN because retry limit was reached before.
 ** @retval 0 on success
 **         -1 on error
 **
 ** This function is equivalent to xos_creduiagent_ask_password, but here 
 ** password is a PIN number and length is the number of digits. Some
 ** creduiagent modules may use the same implementation for the two functions,
 ** that is, ignore in this call that the PIN is a numeric password with a
 ** fixed length. Consider this as a hint, not as a validation.
 **/
int xos_creduiagent_ask_login_pin(CREDUIAGENT_HANDLE handle,char *message,
		char **login,char **pin,int length,char retry);

/** @brief Use creduiagent module to ask a text to user. This text is not
 ** asked as a password: the user may see what he/she type. 
 ** @param[in] handle of creduiagent module. This handle may be obtained
 ** invoking xos_creduiagent_instance and must be destroyed when not needed
 ** anymore with xos_creduiagent_destroy. Handle may be also received as
 ** parameter in a creduiagent modulue function. In last case, don't 
 ** destroy the handle, because handle owner is the caller, not the
 ** creduiagent module.
 ** @param[in] message the informative message to show user when asking him
 **      the code. Pass NULL if you want that the creduiagent shows its default
 **      message (perhaps a message specified in the configuration file)
 ** @param[in] code a pointer to a string that will be filled with the text
 **   provided by user. This string must be freed by the caller.
 ** @param[in] length the maximum number of characters of the code. Warning:
 **            Some creduiagent modules may ignore this parameter. Consider
 **            this value as a hint, not as a validation.
 ** @param[in] retry pass 1 in this parameter if this is invocation is a
 **            retry after user provided a invalid value, pass 0 otherwise.
 **            This is only a hint, some creduiagent modules may use
 **            this value, others may ignore it. 
 ** @retval 0 on success
 ** @retval -1 on error
 **
 ** This function 
 **/
int xos_creduiagent_ask_code(CREDUIAGENT_HANDLE handle,char *message, 
		char **code, int max_length,char retry) ;

/** @brief Ask confirmation to user
 ** @param[in] handle of creduiagent module. This handle may be obtained
 ** invoking xos_creduiagent_instance and must be destroyed when not needed
 ** anymore with xos_creduiagent_destroy. Handle may be also received as
 ** parameter in a creduiagent modulue function. In last case, don't 
 ** destroy the handle, because handle owner is the caller, not the
 ** creduiagent module.
 ** @param[in] message the informative text to display user when asking user
 **      confirmation. Pass NULL if you want that the creduiagent shows its
 **      default message (perhaps a message specified in the configuration file)
 ** @retval 0 on user accept 
 ** @retval -1 on error or on user rejection.
 **
 ** This function is used to get authorization from user to do anything. For
 ** example, some credagent modules using this call to request confirmation from
 ** user that he really want to use a credential, to avoid that a malicious
 ** application stole a credential without user's knowledge.
 **/
int xos_creduiagent_ask_confirmation(CREDUIAGENT_HANDLE handle,char *message) ;

// this function currently is not implemented. We thing is better a
// server side implementation.
int xos_creduiagent_ask_confirmation_pin(CREDUIAGENT_HANDLE handle,
		char *message, char *pin, char *puk,int max_length_pin,
                int max_length_puk, int allowed_retries) ;

/** @brief Show a error message to user using the specified creduiagent.
 ** @param[in] handle of creduiagent module. This handle may be obtained
 ** invoking xos_creduiagent_instance and must be destroyed when not needed
 ** anymore with xos_creduiagent_destroy. Handle may be also received as
 ** parameter in a creduiagent modulue function. In last case, don't 
 ** destroy the handle, because handle owner is the caller, not the
 ** creduiagent module.
 ** @param[in] message the string with the error message to be showed.
 ** @retval 0 on success
 ** @retval -1 on error
 **
 ** This function is used to notify user a error. Actually, creduiagent
 ** module may not display any message: a creduiagent module may choose
 ** for example log the error in a file or ignore the message. 
 **/
int xos_creduiagent_show_error(CREDUIAGENT_HANDLE handle,char *message) ;

/** @brief get a parameter value from "creduiagent" section in configuration file
 ** @param[in] handle of creduiagent module. This handle may be obtained
 ** invoking xos_creduiagent_instance and must be destroyed when not needed
 ** anymore with xos_creduiagent_destroy. Handle may be also received as
 ** parameter in a creduiagent modulue function. In last case, don't 
 ** destroy the handle, because handle owner is the caller, not the
 ** creduiagent module.
 ** @param[in] key the name of the parameter to retrieve from creduiagent section.
 ** @retval string value of the requested parameter on success. The value must be
 **         freed by caller when not needed anymore.
 ** @retval NULL on error
 **
 ** This function is used to obtain a creduiagent parameter from credential
 ** configuration name. Normally, this function is used by creduiagent modules
 ** using the handle received as parameter in the call they provides, not by
 ** credagent modules. Indeed, this is the only libxos-credagent function that
 ** is intended to creduiagent modules.
 **/
char *xos_creduiagent_getparameter(CREDUIAGENT_HANDLE handle,char *key);

/** @brief get a function pointer to a method implemented in creduiagent module
 ** @param[in] handle of creduiagent module. This handle may be obtained
 ** invoking xos_creduiagent_instance and must be destroyed when not needed
 ** anymore with xos_creduiagent_destroy. Handle may be also received as
 ** parameter in a creduiagent modulue function. In last case, don't 
 ** destroy the handle, because handle owner is the caller, not the
 ** creduiagent module.
 ** @param[in] the name of the function to retrieve
 ** @retval fuction pointer to the requested call on success
 ** @retval NULL on error
 **
 ** This helper function allows credagent module programmers to use
 ** functions provided by a creduiagent module that are not standard and
 ** therefore the library does not define specific API to use them.
 **/
void * xos_creduiagent_get_func(CREDUIAGENT_HANDLE handle, char *name);


#endif
